import 'package:flutter/services.dart';

import 'HVMethodCallStrings.dart';

class HyperSnapSDK {
  static const MethodChannel _hyperSnapSDKChannel =
      const MethodChannel(HVMethodCallStrings.HYPERSNAP_SDK_CHANNEL);

  /// initialize() is used initialize the hypersnapsdk inside the client's application
  ///
  /// params :
  ///     appID : type - String, value - Provided by hyperverge.co
  ///     appKey : type - String, value - Provided by hyperverge.co
  ///     region : type - String, value - one of [India, APAC, etc]
  /// returns :
  ///     Future<String> `initialised` if initialization is successful
  static Future<String> initialize(
      String appID, String appKey, String region) async {
    String init = "";
    try {
      init = await _hyperSnapSDKChannel.invokeMethod(
          HVMethodCallStrings.INITIALIZE,
          {"appID": appID, "appKey": appKey, "region": region});
    } catch (e) {
      print(e.toString());
    }

    return init;
  }

  /// initializeWithToken() is used to initilalize the hypersnapsdk with accessToken inside the client's application
  ///
  /// params :
  ///     appID : type - String, value - Provided by hyperverge.co
  ///     accessToken : type - String, value - Provided by hyperverge.co
  ///     region : type - String, value - one of [India, APAC, etc]
  /// returns :
  ///     Future<String> `initialised` if initialization is successful
  static Future<String> initializeWithToken(
      String appID, String accessToken, String region) async {
    String init = "";
    try {
      init = await _hyperSnapSDKChannel
          .invokeMethod(HVMethodCallStrings.INITIALIZE_WITH_TOKEN, {
        "appID": appID,
        "accessToken": accessToken,
        "region": region,
      });
    } catch (e) {
      print(e.toString());
    }
    return init;
  }

  /// setShouldEnableSSLPinning() is set to true if ssl pinning has to enabled in the application
  ///
  /// params :
  ///     shouldEnableSSLPinning : type - bool, value - true/false
  /// returns :
  ///     void
  static setShouldEnableSSLPinning(bool shouldEnableSSLPinning) async {
    try {
      await _hyperSnapSDKChannel.invokeMethod(
          HVMethodCallStrings.SET_SHOULD_ENABLE_SSL_PINNING,
          {"shouldEnableSSLPinning": shouldEnableSSLPinning});
    } catch (e) {
      print(e.toString());
    }
  }

  /// setShouldUseSignature() is set to true if signature of the images has to be used while involving api calls
  /// in the application
  ///
  /// params :
  ///     shouldUseSignature : type - bool
  /// returns :
  ///     void
  static setShouldUseSignature(bool shouldUseSignature) async {
    try {
      await _hyperSnapSDKChannel.invokeMethod(
          HVMethodCallStrings.SET_SHOULD_USE_SIGNATURE,
          {"shouldUseSignature": shouldUseSignature});
    } catch (e) {
      print(e.toString());
    }
  }

  /// setHttpTimeoutValues() is used to set connectionTimeout, readTimeout, writeTimeout for the api calls
  ///
  /// params :
  ///     connectTimeout : type - int
  ///     readTimeout : type - int
  ///     writeTimeout : type - int
  /// returns :
  ///     void
  static setHttpTimeoutValues(
      int connectTimeout, int readTimeout, int writeTimeout) async {
    try {
      await _hyperSnapSDKChannel
          .invokeMethod(HVMethodCallStrings.SET_HTTPS_TIMEOUT_VALUES, {
        "connectTimeout": connectTimeout,
        "readTimeout": readTimeout,
        "writeTimeout": writeTimeout
      });
    } catch (e) {
      print(e.toString());
    }
  }

  /// sortJSONKeysAlphabetically() is used to sort the keys of the response object alphabetically so that signature checks
  /// and other related checks can be performed in a better/faster way
  ///
  /// params :
  ///     map : type - Map, value - response object
  /// returns :
  ///     Future<String>
  static Future<String> sortJSONKeysAlphabetically(Map map) async {
    String string = "";
    try {
      string = await _hyperSnapSDKChannel.invokeMethod(
          HVMethodCallStrings.SORT_JSON_KEYS_ALPHABETICALLY, {"map": map});
    } catch (e) {
      print(e.toString());
    }
    return string;
  }

  /// setShouldReturnRawResponse() is set to true if raw response has to be returned from the plugin
  ///
  /// params :
  ///     shouldReturnRawResponse : type - bool
  /// returns :
  ///     void
  static setShouldReturnRawResponse(bool shouldReturnRawResponse) async {
    try {
      await _hyperSnapSDKChannel.invokeMethod(
          HVMethodCallStrings.SET_SHOULD_RETURN_RAW_RESPONSE,
          {"shouldReturnRawResponse": shouldReturnRawResponse});
    } catch (e) {
      print(e.toString());
    }
  }

  /// startUserSession() is used to start the user session in the application
  ///
  /// params :
  ///     userID : type - String, value - userID which can be generated by the client
  /// returns :
  ///    Future<bool>, returns true if user session has been started successfully
  static Future<bool> startUserSession(String userID) async {
    bool flag = false;
    try {
      flag = await _hyperSnapSDKChannel.invokeMethod(
          HVMethodCallStrings.START_USER_SESSION, {"userID": userID});
    } catch (e) {
      print(e.toString());
    }
    return flag;
  }

  /// endUserSession() is used by the client to end the session if any session is active
  ///
  /// params : empty
  /// returns : void
  static endUserSession() async {
    try {
      await _hyperSnapSDKChannel
          .invokeMethod(HVMethodCallStrings.END_USER_SESSION);
    } catch (e) {
      print(e.toString());
    }
  }
}
